@@ -4,17 +4,15 @@ Get API information encoded in C files.
 See ``find_function`` for how functions should be formatted, and
 ``read_order`` for how the order of the functions should be
 specified.
-"""
-import sys, os, re
-try:
-    import hashlib
-    md5new = hashlib.md5
-except ImportError:
-    import md5
-    md5new = md5.new
-if sys.version_info[:2] < (2, 6):
-    from sets import Set as set
 
+"""
+from numpy.distutils.conv_template import process_file as process_c_file
+
+import hashlib
+import io
+import os
+import re
+import sys
 import textwrap
 
 from os.path import join
@@ -22,31 +20,53 @@ from os.path import join
 __docformat__ = 'restructuredtext'
 
 # The files under src/ that are scanned for API functions
-API_FILES = [join('multiarray', 'methods.c'),
+API_FILES = [join('multiarray', 'alloc.c'),
+             join('multiarray', 'abstractdtypes.c'),
+             join('multiarray', 'arrayfunction_override.c'),
+             join('multiarray', 'array_assign_array.c'),
+             join('multiarray', 'array_assign_scalar.c'),
+             join('multiarray', 'array_coercion.c'),
+             join('multiarray', 'array_method.c'),
              join('multiarray', 'arrayobject.c'),
-             join('multiarray', 'flagsobject.c'),
-             join('multiarray', 'descriptor.c'),
-             join('multiarray', 'iterators.c'),
-             join('multiarray', 'getset.c'),
-             join('multiarray', 'number.c'),
-             join('multiarray', 'sequence.c'),
-             join('multiarray', 'ctors.c'),
-             join('multiarray', 'convert.c'),
-             join('multiarray', 'shape.c'),
-             join('multiarray', 'item_selection.c'),
-             join('multiarray', 'convert_datatype.c'),
              join('multiarray', 'arraytypes.c.src'),
+             join('multiarray', 'buffer.c'),
+             join('multiarray', 'calculation.c'),
+             join('multiarray', 'common_dtype.c'),
+             join('multiarray', 'conversion_utils.c'),
+             join('multiarray', 'convert.c'),
+             join('multiarray', 'convert_datatype.c'),
+             join('multiarray', 'ctors.c'),
+             join('multiarray', 'datetime.c'),
+             join('multiarray', 'datetime_busday.c'),
+             join('multiarray', 'datetime_busdaycal.c'),
+             join('multiarray', 'datetime_strings.c'),
+             join('multiarray', 'descriptor.c'),
+             join('multiarray', 'dlpack.c'),
+             join('multiarray', 'dtypemeta.c'),
+             join('multiarray', 'einsum.c.src'),
+             join('multiarray', 'flagsobject.c'),
+             join('multiarray', 'getset.c'),
+             join('multiarray', 'item_selection.c'),
+             join('multiarray', 'iterators.c'),
+             join('multiarray', 'mapping.c'),
+             join('multiarray', 'methods.c'),
              join('multiarray', 'multiarraymodule.c'),
+             join('multiarray', 'nditer_api.c'),
+             join('multiarray', 'nditer_constr.c'),
+             join('multiarray', 'nditer_pywrap.c'),
+             join('multiarray', 'nditer_templ.c.src'),
+             join('multiarray', 'number.c'),
+             join('multiarray', 'refcount.c'),
              join('multiarray', 'scalartypes.c.src'),
              join('multiarray', 'scalarapi.c'),
-             join('multiarray', 'calculation.c'),
+             join('multiarray', 'sequence.c'),
+             join('multiarray', 'shape.c'),
+             join('multiarray', 'strfuncs.c'),
              join('multiarray', 'usertypes.c'),
-             join('multiarray', 'refcount.c'),
-             join('multiarray', 'conversion_utils.c'),
-             join('multiarray', 'buffer.c'),
-             join('multiarray', 'datetime.c'),
-             join('umath', 'ufunc_object.c'),
              join('umath', 'loops.c.src'),
+             join('umath', 'ufunc_object.c'),
+             join('umath', 'ufunc_type_resolution.c'),
+             join('umath', 'reduction.c'),
             ]
 THIS_DIR = os.path.dirname(__file__)
 API_FILES = [os.path.join(THIS_DIR, '..', 'src', a) for a in API_FILES]
@@ -58,23 +78,46 @@ def remove_whitespace(s):
     return ''.join(s.split())
 
 def _repl(str):
-    return str.replace('intp', 'npy_intp').replace('Bool','npy_bool')
+    return str.replace('Bool', 'npy_bool')
 
-class Function(object):
+
+class StealRef:
+    def __init__(self, arg):
+        self.arg = arg # counting from 1
+
+    def __str__(self):
+        try:
+            return ' '.join('NPY_STEALS_REF_TO_ARG(%d)' % x for x in self.arg)
+        except TypeError:
+            return 'NPY_STEALS_REF_TO_ARG(%d)' % self.arg
+
+
+class NonNull:
+    def __init__(self, arg):
+        self.arg = arg # counting from 1
+
+    def __str__(self):
+        try:
+            return ' '.join('NPY_GCC_NONNULL(%d)' % x for x in self.arg)
+        except TypeError:
+            return 'NPY_GCC_NONNULL(%d)' % self.arg
+
+
+class Function:
     def __init__(self, name, return_type, args, doc=''):
         self.name = name
         self.return_type = _repl(return_type)
         self.args = args
         self.doc = doc
 
-    def _format_arg(self, (typename, name)):
+    def _format_arg(self, typename, name):
         if typename.endswith('*'):
             return typename + name
         else:
             return typename + ' ' + name
 
     def __str__(self):
-        argstr = ', '.join([self._format_arg(a) for a in self.args])
+        argstr = ', '.join([self._format_arg(*a) for a in self.args])
         if self.doc:
             doccomment = '/* %s */\n' % self.doc
         else:
@@ -83,7 +126,7 @@ class Function(object):
 
     def to_ReST(self):
         lines = ['::', '', '  ' + self.return_type]
-        argstr = ',\000'.join([self._format_arg(a) for a in self.args])
+        argstr = ',\000'.join([self._format_arg(*a) for a in self.args])
         name = '  %s' % (self.name,)
         s = textwrap.wrap('(%s)' % (argstr,), width=72,
                           initial_indent=name,
@@ -97,7 +140,7 @@ class Function(object):
         return '\n'.join(lines)
 
     def api_hash(self):
-        m = md5new()
+        m = hashlib.md5()
         m.update(remove_whitespace(self.return_type))
         m.update('\000')
         m.update(self.name)
@@ -129,14 +172,12 @@ def skip_brackets(s, lbrac, rbrac):
 
 def split_arguments(argstr):
     arguments = []
-    bracket_counts = {'(': 0, '[': 0}
     current_argument = []
-    state = 0
     i = 0
     def finish_arg():
         if current_argument:
             argstr = ''.join(current_argument).strip()
-            m = re.match(r'(.*(\s+|[*]))(\w+)$', argstr)
+            m = re.match(r'(.*(\s+|\*))(\w+)$', argstr)
             if m:
                 typename = m.group(1).strip()
                 name = m.group(3)
@@ -182,13 +223,16 @@ def find_functions(filename, tag='API'):
           This function does foo...
          */
     """
-    fo = open(filename, 'r')
+    if filename.endswith(('.c.src', '.h.src')):
+        fo = io.StringIO(process_c_file(filename))
+    else:
+        fo = open(filename, 'r')
     functions = []
     return_type = None
     function_name = None
     function_args = []
     doclist = []
-    SCANNING, STATE_DOC, STATE_RETTYPE, STATE_NAME, STATE_ARGS = range(5)
+    SCANNING, STATE_DOC, STATE_RETTYPE, STATE_NAME, STATE_ARGS = list(range(5))
     state = SCANNING
     tagcomment = '/*' + tag
     for lineno, line in enumerate(fo):
@@ -226,7 +270,8 @@ def find_functions(filename, tag='API'):
             elif state == STATE_ARGS:
                 if line.startswith('{'):
                     # finished
-                    fargs_str = ' '.join(function_args).rstrip(' )')
+                    # remove any white space and the closing bracket:
+                    fargs_str = ' '.join(function_args).rstrip()[:-1].rstrip()
                     fargs = split_arguments(fargs_str)
                     f = Function(function_name, return_type, fargs,
                                  '\n'.join(doclist))
@@ -238,9 +283,11 @@ def find_functions(filename, tag='API'):
                     state = SCANNING
                 else:
                     function_args.append(line)
-        except:
-            print filename, lineno+1
+        except ParseError:
             raise
+        except Exception as e:
+            msg = "see chained exception for details"
+            raise ParseError(filename, lineno + 1, msg) from e
     fo.close()
     return functions
 
@@ -254,13 +301,29 @@ def should_rebuild(targets, source_files):
         return True
     return False
 
+def write_file(filename, data):
+    """
+    Write data to filename
+    Only write changed data to avoid updating timestamps unnecessarily
+    """
+    if os.path.exists(filename):
+        with open(filename) as f:
+            if data == f.read():
+                return
+
+    with open(filename, 'w') as fid:
+        fid.write(data)
+
+
 # Those *Api classes instances know how to output strings for the generated code
 class TypeApi:
-    def __init__(self, name, index, ptr_cast, api_name):
+    def __init__(self, name, index, ptr_cast, api_name, internal_type=None):
         self.index = index
         self.name = name
         self.ptr_cast = ptr_cast
         self.api_name = api_name
+        # The type used internally, if None, same as exported (ptr_cast)
+        self.internal_type = internal_type
 
     def define_from_array_api_string(self):
         return "#define %s (*(%s *)%s[%d])" % (self.name,
@@ -272,13 +335,19 @@ class TypeApi:
         return "        (void *) &%s" % self.name
 
     def internal_define(self):
-        astr = """\
-#ifdef NPY_ENABLE_SEPARATE_COMPILATION
-    extern NPY_NO_EXPORT PyTypeObject %(type)s;
-#else
-    NPY_NO_EXPORT PyTypeObject %(type)s;
-#endif
-""" % {'type': self.name}
+        if self.internal_type is None:
+            return f"extern NPY_NO_EXPORT {self.ptr_cast} {self.name};\n"
+
+        # If we are here, we need to define a larger struct internally, which
+        # the type can be cast safely. But we want to normally use the original
+        # type, so name mangle:
+        mangled_name = f"{self.name}Full"
+        astr = (
+            # Create the mangled name:
+            f"extern NPY_NO_EXPORT {self.internal_type} {mangled_name};\n"
+            # And define the name as: (*(type *)(&mangled_name))
+            f"#define {self.name} (*({self.ptr_cast} *)(&{mangled_name}))\n"
+        )
         return astr
 
 class GlobalVarApi:
@@ -299,11 +368,7 @@ class GlobalVarApi:
 
     def internal_define(self):
         astr = """\
-#ifdef NPY_ENABLE_SEPARATE_COMPILATION
-    extern NPY_NO_EXPORT %(type)s %(name)s;
-#else
-    NPY_NO_EXPORT %(type)s %(name)s;
-#endif
+extern NPY_NO_EXPORT %(type)s %(name)s;
 """ % {'type': self.type, 'name': self.name}
         return astr
 
@@ -327,18 +392,15 @@ class BoolValuesApi:
 
     def internal_define(self):
         astr = """\
-#ifdef NPY_ENABLE_SEPARATE_COMPILATION
 extern NPY_NO_EXPORT PyBoolScalarObject _PyArrayScalar_BoolValues[2];
-#else
-NPY_NO_EXPORT PyBoolScalarObject _PyArrayScalar_BoolValues[2];
-#endif
 """
         return astr
 
 class FunctionApi:
-    def __init__(self, name, index, return_type, args, api_name):
+    def __init__(self, name, index, annotations, return_type, args, api_name):
         self.name = name
         self.index = index
+        self.annotations = annotations
         self.return_type = return_type
         self.args = args
         self.api_name = api_name
@@ -363,18 +425,20 @@ class FunctionApi:
         return "        (void *) %s" % self.name
 
     def internal_define(self):
+        annstr = [str(a) for a in self.annotations]
+        annstr = ' '.join(annstr)
         astr = """\
-NPY_NO_EXPORT %s %s \\\n       (%s);""" % (self.return_type,
-                                           self.name,
-                                           self._argtypes_string())
+NPY_NO_EXPORT %s %s %s \\\n       (%s);""" % (annstr, self.return_type,
+                                              self.name,
+                                              self._argtypes_string())
         return astr
 
 def order_dict(d):
     """Order dict by its values."""
-    o = d.items()
-    def cmp(x, y):
-        return x[1] - y[1]
-    return sorted(o, cmp=cmp)
+    o = list(d.items())
+    def _key(x):
+        return x[1] + (x[0],)
+    return sorted(o, key=_key)
 
 def merge_api_dicts(dicts):
     ret = {}
@@ -386,28 +450,32 @@ def merge_api_dicts(dicts):
 
 def check_api_dict(d):
     """Check that an api dict is valid (does not use the same index twice)."""
+    # remove the extra value fields that aren't the index
+    index_d = {k: v[0] for k, v in d.items()}
+
     # We have if a same index is used twice: we 'revert' the dict so that index
     # become keys. If the length is different, it means one index has been used
     # at least twice
-    revert_dict = dict([(v, k) for k, v in d.items()])
-    if not len(revert_dict) == len(d):
+    revert_dict = {v: k for k, v in index_d.items()}
+    if not len(revert_dict) == len(index_d):
         # We compute a dict index -> list of associated items
         doubled = {}
-        for name, index in d.items():
+        for name, index in index_d.items():
             try:
                 doubled[index].append(name)
             except KeyError:
                 doubled[index] = [name]
-        msg = """\
-Same index has been used twice in api definition: %s
-""" % ['index %d -> %s' % (index, names) for index, names in doubled.items() \
-                                          if len(names) != 1]
-        raise ValueError(msg)
+        fmt = "Same index has been used twice in api definition: {}"
+        val = ''.join(
+            '\n\tindex {} -> {}'.format(index, names)
+            for index, names in doubled.items() if len(names) != 1
+        )
+        raise ValueError(fmt.format(val))
 
     # No 'hole' in the indexes may be allowed, and it must starts at 0
-    indexes = set(d.values())
+    indexes = set(index_d.values())
     expected = set(range(len(indexes)))
-    if not indexes == expected:
+    if indexes != expected:
         diff = expected.symmetric_difference(indexes)
         msg = "There are some holes in the API indexing: " \
               "(symmetric diff is %s)" % diff
@@ -418,10 +486,7 @@ def get_api_functions(tagname, api_dict):
     functions = []
     for f in API_FILES:
         functions.extend(find_functions(f, tagname))
-    dfunctions = []
-    for func in functions:
-        o = api_dict[func.name]
-        dfunctions.append( (o, func) )
+    dfunctions = [(api_dict[func.name][0], func) for func in functions]
     dfunctions.sort()
     return [a[1] for a in dfunctions]
 
@@ -430,32 +495,25 @@ def fullapi_hash(api_dicts):
     of the list of items in the API (as a string)."""
     a = []
     for d in api_dicts:
-        def sorted_by_values(d):
-            """Sort a dictionary by its values. Assume the dictionary items is of
-            the form func_name -> order"""
-            return sorted(d.items(), key=lambda (x, y): (y, x))
-        for name, index in sorted_by_values(d):
+        for name, data in order_dict(d):
             a.extend(name)
-            a.extend(str(index))
+            a.extend(','.join(map(str, data)))
 
-    return md5new(''.join(a)).hexdigest()
+    return hashlib.md5(''.join(a).encode('ascii')).hexdigest()
 
 # To parse strings like 'hex = checksum' where hex is e.g. 0x1234567F and
 # checksum a 128 bits md5 checksum (hex format as well)
-VERRE = re.compile('(^0x[\da-f]{8})\s*=\s*([\da-f]{32})')
+VERRE = re.compile(r'(^0x[\da-f]{8})\s*=\s*([\da-f]{32})')
 
 def get_versions_hash():
     d = []
 
     file = os.path.join(os.path.dirname(__file__), 'cversions.txt')
-    fid = open(file, 'r')
-    try:
-        for line in fid.readlines():
+    with open(file, 'r') as fid:
+        for line in fid:
             m = VERRE.match(line)
             if m:
                 d.append((int(m.group(1), 16), m.group(2)))
-    finally:
-        fid.close()
 
     return dict(d)
 
@@ -463,13 +521,13 @@ def main():
     tagname = sys.argv[1]
     order_file = sys.argv[2]
     functions = get_api_functions(tagname, order_file)
-    m = md5new(tagname)
+    m = hashlib.md5(tagname)
     for func in functions:
-        print func
+        print(func)
         ah = func.api_hash()
         m.update(ah)
-        print hex(int(ah,16))
-    print hex(int(m.hexdigest()[:8],16))
+        print(hex(int(ah, 16)))
+    print(hex(int(m.hexdigest()[:8], 16)))
 
 if __name__ == '__main__':
     main()
