@@ -1,5 +1,5 @@
-#ifndef _NPY_SCALARTYPES_H_
-#define _NPY_SCALARTYPES_H_
+#ifndef NUMPY_CORE_SRC_MULTIARRAY_SCALARTYPES_H_
+#define NUMPY_CORE_SRC_MULTIARRAY_SCALARTYPES_H_
 
 /* Internal look-up tables */
 extern NPY_NO_EXPORT unsigned char
@@ -19,16 +19,8 @@ initialize_casting_tables(void);
 NPY_NO_EXPORT void
 initialize_numeric_types(void);
 
-NPY_NO_EXPORT void
-format_longdouble(char *buf, size_t buflen, npy_longdouble val, unsigned int prec);
-
-#if PY_VERSION_HEX >= 0x03000000
 NPY_NO_EXPORT void
 gentype_struct_free(PyObject *ptr);
-#else
-NPY_NO_EXPORT void
-gentype_struct_free(void *ptr, void *arg);
-#endif
 
 NPY_NO_EXPORT int
 is_anyscalar_exact(PyObject *obj);
@@ -39,4 +31,4 @@ _typenum_fromtypeobj(PyObject *type, int user);
 NPY_NO_EXPORT void *
 scalar_value(PyObject *scalar, PyArray_Descr *descr);
 
-#endif
+#endif  /* NUMPY_CORE_SRC_MULTIARRAY_SCALARTYPES_H_ */
